/*
 * File:        design-core.js
 * CVS:         $Id$
 * Description: Common core functions used for Design
 * Author:			Allan Jardine
 * Created:		  28-11-07
 * Modified:		$Date$ by $Author$
 * Language:		JavaScript
 * Project:		  Design
 * License:     GNU GPL v2.0
 * 
 * Copyright 2007-2008 Allan Jardine. All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * version 2.0 as published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

/*
 * This file is a stripped down version of the DesignCore used for 'Design'
 * by Allan Jardine (www.sprymedia.co.uk).
 */

if ( typeof DesignCore != 'undefined' )
{
	DesignCore = undefined;
}


/*
 * Variable: DesignCore
 * Purpose:	 Container object for Rule controller
 * Scope:	   Global
 */
var DesignCore = {
	
	/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
	 * General
	 *
	 * Function: DesignCore.fnGetElementsByClassName
	 * Purpose:  Pull an array of nodes out of the DOM tree of a particular class
	 * Returns:  node array - array of DOM nodes
	 * Inputs:   sClassname - the class to be looked for
	 *           sTag - optional - tag names to be sorted sorted through
	 *           nNode - optional - a node for which all the child elements are 
	 *                   sorted
	 */
	fnGetElementsByClassName: function ( sClassname, sTag, nNode )
	{
		if ( sTag == null )
			sTag = '*';
		if ( nNode == null )
			nNode = document;
		
		var aReturn = new Array ();
		var sRegExp = new RegExp('\\b' + sClassname + '\\b');
		var nElements = nNode.getElementsByTagName(sTag);
		for ( var i=0, j=nElements.length ; i<j ; i++ )
		{
			if ( sRegExp.test(nElements[i].className) ) {
				aReturn.push ( nElements[i] );
			}
		}
		return ( aReturn );
	},
	
	
	/*
	 * Function: DesignCore.fnCreateCookie
	 * Purpose:  Create a new cookie with a value to store the state of Grid
	 * Returns:  
	 * Inputs:   string:sName - name of the cookie
	 *           string:sValue - value the cookie should take
	 * Notes:    -
	 */
	fnCreateCookie: function ( sName, sValue )
	{
		var iDays = 365;
		var date = new Date();
		date.setTime( date.getTime()+(iDays*24*60*60*1000) );
		var sExpires = "; expires="+date.toGMTString();
		
		document.cookie = sName+"="+sValue+sExpires+"; path=/";
	},
	
	
	/*
	 * Function: DesignCore.fnReadCookie
	 * Purpose:  Read an old cookie to get a cookie with an old Grid state
	 * Returns:  string: the cookie contents - or null
	 * Inputs:   string:sName - name of the cookie
	 * Notes:    -
	 */
	fnReadCookie: function ( sName )
	{
		var sNameEQ = sName + "=";
		var sCookieContents = document.cookie.split(';');
		
		for( var i=0 ; i<sCookieContents.length ; i++ )
		{
			var c = sCookieContents[i];
			
			while (c.charAt(0)==' ')
			{
				c = c.substring(1,c.length);
			}
			
			if (c.indexOf(sNameEQ) == 0)
			{
				return c.substring(sNameEQ.length,c.length);
			}
		}
		
		return null;
	},
	
	
	
	/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
	 * 3rd party
	 *
	 * Soure:   json.org
	 * License: Public domain
	
	Copyright (c) 2002 JSON.org
	
	Permission is hereby granted, free of charge, to any person obtaining a copy 
	of this software and associated documentation files (the "Software"), to deal 
	in the Software without restriction, including without limitation the rights 
	to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
	copies of the Software, and to permit persons to whom the Software is 
	furnished to do so, subject to the following conditions:
	
	The above copyright notice and this permission notice shall be included in all 
	copies or substantial portions of the Software.
	
	The Software shall be used for Good, not Evil.
	
	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
	IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
	FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
	AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
	LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
	OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE 
	SOFTWARE.
	*/
	
	toJsonString: function(arg) {
	    return this.toJsonStringArray(arg).join('');
	},
	
	toJsonStringArray: function(arg, out) {
	    out = out || new Array();
	    var u; // undefined
	
	    switch (typeof arg) {
	    case 'object':
	        if (arg) {
	            if (arg.constructor == Array) {
	                out.push('[');
	                for (var i = 0; i < arg.length; ++i) {
	                    if (i > 0)
	                        out.push(',');
	                    this.toJsonStringArray(arg[i], out);
	                }
	                out.push(']');
	                return out;
	            } else if (typeof arg.toString != 'undefined') {
	                out.push('{');
	                var first = true;
	                for (var i in arg) {
	                    var curr = out.length; // Record position to allow undo when arg[i] is undefined.
	                    if (!first)
	                        out.push(',');
	                    this.toJsonStringArray(i, out);
	                    out.push(':');                    
	                    this.toJsonStringArray(arg[i], out);
	                    if (out[out.length - 1] == u)
	                        out.splice(curr, out.length - curr);
	                    else
	                        first = false;
	                }
	                out.push('}');
	                return out;
	            }
	            return out;
	        }
	        out.push('null');
	        return out;
	    case 'unknown':
	    case 'undefined':
	    case 'function':
	        out.push(u);
	        return out;
	    case 'string':
	        out.push('"')
	        out.push(arg.replace(/(["\\])/g, '\\$1').replace(/\r/g, '').replace(/\n/g, '\\n'));
	        out.push('"');
	        return out;
	    default:
	        out.push(String(arg));
	        return out;
	    }
	}
}