/*
 * File:        label.js
 * CVS:         $Id$
 * Description: Label display for web-pages
 * Author:			Allan Jardine
 * Created:		  9-12-07
 * Modified:		$Date$ by $Author$
 * Language:		JavaScript
 * Project:		  Design
 * License:     GNU GPL v2.0
 * 
 * Copyright 2007-2008 Allan Jardine. All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * version 2.0 as published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */


/* Check to see if we have Label already - if so then use the install already
 * available to add a new label to the page
 */
if ( typeof Label != 'undefined' && 
     typeof Label.fnCreateLabel != 'undefined' )
{
	Label.fnCreateLabel();
}
else
{

/*
 * Variable: Label
 * Purpose:	 Container object for page labels
 * Scope:	   Global
 */
var Label = {
	/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
	 * Vars
	 */
	
	/*
	 * Variable: oDrag
	 * Purpose:	 Store information relating to the draging of an object
	 * Scope:	   Label
	 * Notes:	   Object with the following properties:
	 *   node:nTarget - The node to move
	 *   int:iLabelIndex - The index of the label in aoLabels which we are using
	 *   int:iNodeStartX - Mouse down position of the node (x)
	 *   int:iNodeStartY - Mouse down position of the node (y)
	 *   int:iMouseStartX - Mouse down position of the mouse (x)
	 *   int:iMouseStartY - Mouse down position of the mouse (y)
	 */
	oDrag: {
		sType: null, // Mouse move type - 'resize', 'move'
		nTarget: null,
		iLabelIndex: null,
		iNodeStartX: null,
		iNodeStartY: null,
		iNodeStartHeight: null,
		iNodeStartWidth: null,
		iMouseStartX: null,
		iMouseStartY: null
	},
	
	/*
	 * Variable: aoLabels
	 * Purpose:	 State information of each label on the page
	 * Scope:	   Label
	 * Notes:	   Array of object with the following properties:
	 *   string:sId - ID of the label in the DOM
	 *   string:sContent
	 *   string:sColour
	 *   int:iSize
	 *   int:iTop
	 *   int:iLeft
	 *   int:iWidth
	 *   int:iHeight
	 */
	aoLabels: new Array(),
	
	/*
	 * Variable: sUserShareCode
	 * Purpose:	 Sharing code for the labels on this page
	 * Scope:	   Label
	 * Notes:	   Code the user can enter to save information for another time
	 *   or share with other people if they wish.
	 */
	sUserShareCode: '',
	
	/*
	 * Variable: sBrowserStoreCode
	 * Purpose:	 Store code for the labels on this page
	 * Scope:	   Label
	 * Notes:	   This is used by the backend when the user hasn't specificed 
	 *   a sharing code. This is so there are no collisions between different
	 *   user's Labels.
	 */
	sBrowserStoreCode: '',
	
	/*
	 * Variable: toSaveTimer
	 * Purpose:	 'Gloabl' timer object for auto saving
	 * Scope:	   Label
	 */
	toSaveTimer: null,
	
	/*
	 * Variable: iSaveTimer
	 * Purpose:	 Auto save delay - milliseconds
	 * Scope:	   Label
	 */
	iSaveTimer: 2000,
	
	/*
	 * Variable: iTopZIndex
	 * Purpose:	 Cache the top z=index value
	 * Scope:	   Label
	 */
	iTopZIndex: 33000,
	
	
	/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
	 * Init
	 */
	
	/*
	 * Function: fnPrerequisites
	 * Purpose:  Change that all we need is able and then go!
	 * Returns:  -
	 * Inputs:   -
	 */
	fnPrerequisites: function ()
	{
		var bGo = false;
		
		// Check we've got the objects we need
		if ( typeof jsCore == 'object' && typeof DesignCore == 'object' &&
		     typeof Remoting == 'object' )
		{
			bGo = true;
		}
		
		bGo ?
			this.fnInit() :
			setTimeout( function(){ Label.fnPrerequisites(); }, 50 );
	},
	
	/*
	 * Function: fnInit
	 * Purpose:  Initialisation of Labels
	 * Returns:  -
	 * Inputs:   -
	 */
	fnInit: function ()
	{
		/* Read the cookie with the browser code if there is one */
		var oObj = eval( '('+DesignCore.fnReadCookie( 'SpryMediaUK_Label' )+')' );
		if ( oObj != null )
		{
			this.sBrowserStoreCode = oObj.sBrowserStoreCode;
			this.sUserShareCode = oObj.sUserShareCode;
		}
		
		/* If there is a global share code variable - override cookie options */
		if ( typeof gsShareCode != 'undefined' )
		{
			this.sUserShareCode = gsShareCode;
		}
		
		/* Try to load a state */
		if ( this.fnStateLoad() == -1 )
		{
			/* No saved state - so let's make a new one */
			this.sBrowserStoreCode = this.fnGenerateBrowserCode();
			this.fnCreateLabel();
			
			if ( document.getElementById('LoadingLabel') )
			{
				document.body.removeChild( document.getElementById('LoadingLabel') );
			}
		}
	},
	
	
	/*
	 * Function: fnCreateLabel
	 * Purpose:  Add the HTML for a label to the DOM
	 * Returns:  -
	 * Inputs:   int:iLabelIndex - Index of the label object
	 *           int:iParentLabel - Index of the new label's parent - optional
	 */
	fnCreateLabel: function ( iLabelIndex, iParentLabel )
	{
		if ( iLabelIndex == undefined || iLabelIndex == null )
		{
			iLabelIndex = this.aoLabels.length;
			
			this.aoLabels[this.aoLabels.length++] = this.fnCreateLabelObject( 
				typeof iParentLabel != 'undefined' ? iParentLabel : null );
		}
		
		var nSticky = document.createElement( 'div' );
		nSticky.setAttribute( 'id', this.aoLabels[iLabelIndex].sId );
		nSticky.className = 'label_content_wrapper '+
			'label_'+this.aoLabels[iLabelIndex].sColour;
		nSticky.style.top = this.aoLabels[iLabelIndex].iTop;
		nSticky.style.left = this.aoLabels[iLabelIndex].iLeft;
		nSticky.style.width = this.aoLabels[iLabelIndex].iWidth;
		nSticky.style.height = this.aoLabels[iLabelIndex].iHeight;
		nSticky.style.zIndex = this.aoLabels[iLabelIndex].iZIndex;
		
		var sHtml = '\
			<div class="label_background"></div>\
			<div class="label_left"></div>\
			<div class="label_right"></div>\
			<div class="label_top"></div>\
			<div class="label_bottom_left"></div>\
			<div class="label_bottom_right"></div>\
			\
			<div class="label_top_left_corner"></div>\
			<div class="label_top_right_corner"></div>\
			<div class="label_bottom_left_corner"></div>\
			<div class="label_bottom_right_corner"></div>\
			\
			<div class="label_content">\
				<textarea></textarea>\
			</div>'
		
		
		/* if there is a global share code - don't show the load and share 
		 * options
		 */
		if ( typeof gsShareCode == 'undefined' )
		{
			sHtml += '\
				<div class="label_info">\
					<div class="label_info_details">&nbsp;</div>\
					<a href="New" class="label_info_new">New</a>&nbsp;&nbsp;'+
					'<a href="Load" class="label_info_load">Load</a>&nbsp;&nbsp;'+
					'<a href="Share" class="label_info_share">Share</a>&nbsp;&nbsp;'+
					'<a href="Options" class="label_info_options">Options</a>&nbsp;&nbsp;'+
					'<a href="Delete" class="label_info_delete">Delete</a>\
				</div>';
		}
		else
		{
			sHtml += '\
				<div class="label_info">\
					<div class="label_info_details">&nbsp;</div>\
					<a href="New" class="label_info_new">New</a>&nbsp;&nbsp;'+
					'<a href="Options" class="label_info_options">Options</a>&nbsp;&nbsp;'+
					'<a href="Delete" class="label_info_delete">Delete</a>\
				</div>';
		}
		
		sHtml += '<div class="label_resize"></div>';
		
		
		nSticky.innerHTML = sHtml;
		document.body.appendChild( nSticky );
		
		/* Movement */
		jsCore.addListener( {
			'mElement': nSticky,
			'sType': 'mousedown',
			'fnCallback': this.fnMouseDown,
			'oObj': this,
			'mScope': true,
			'sLabel': 'Label' } );
		
		jsCore.addListener( {
			'mElement': DesignCore.fnGetElementsByClassName('label_title', 'div', nSticky ),
			'sType': 'mousedown',
			'fnCallback': this.fnMouseDown,
			'oObj': this,
			'mScope': true,
			'sLabel': 'Label' } );
		
		/* Resize */
		jsCore.addListener( {
			'mElement': DesignCore.fnGetElementsByClassName('label_resize', 'div', nSticky ),
			'sType': 'mousedown',
			'fnCallback': this.fnMouseDown,
			'oObj': this,
			'mScope': true,
			'sLabel': 'Label' } );
		
		/* Controls */
		jsCore.addListener( {
			'mElement': nSticky.getElementsByTagName('a'),
			'sType': 'click',
			'fnCallback': this.fnDetails,
			'oObj': this,
			'mScope': true,
			'sLabel': 'Label' } );
		
		/* Textarea */
		this.fnInitTextarea( nSticky.getElementsByTagName('textarea')[0], iLabelIndex );
	},
	
	
	/*
	 * Function: fnInitTextarea
	 * Purpose:  Init an textarea node for Label
	 * Returns:  -
	 * Inputs:   node:nTextarea - the textarea tag that should be initialised
	 *           int:iLabelIndex - Index of the label object
	 */
	fnInitTextarea: function ( nTextarea, iLabelIndex )
	{
		// Add some content
		nTextarea.value = this.aoLabels[iLabelIndex].sContent;
		nTextarea.className = nTextarea.parentNode.parentNode.getAttribute('id');
		
		// Add a few styles
		nTextarea.style.fontSize = this.aoLabels[iLabelIndex].iSize+'px';
		nTextarea.style.lineHeight = "1.3em";
		
		/* Save timer */
		jsCore.addListener( {
			'mElement': nTextarea,
			'sType': 'keyup',
			'fnCallback': this.fnContentKeyUp,
			'oObj': this,
			'mScope': true,
			'sLabel': 'Label' } );
	},
	
	
	
	
	/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
	 * State control
	 */
	
	/*
	 * Function: fnStateSave
	 * Purpose:  Save the state of Label
	 * Returns:  -
	 * Inputs:   bool:bNow - save right away or have a pause
	 */
	fnStateSave: function ( bNow )
	{
		/* Clear any previous timer so we don't hammer the sever */
		clearTimeout( this.toSaveTimer );
		
		if ( bNow )
		{
			this.fnStateServerSave();
		}
		else
		{
			this.toSaveTimer = setTimeout( 
				function(){ Label.fnStateServerSave.call( Label ); }, 
				this.iSaveTimer );
		}
		
		// Ensure the cookie has the right info
		var oObj = {
			'sBrowserStoreCode': this.sBrowserStoreCode,
			'sUserShareCode': this.sUserShareCode };
		
		DesignCore.fnCreateCookie( 'SpryMediaUK_Label', 
			DesignCore.toJsonString( oObj ) );
	},
	
	
	/*
	 * Function: fnStateServerSave
	 * Purpose:  Save the state of Label to the server
	 * Returns:  -
	 * Inputs:   -
	 * Notes:    All browsers (partly due to negative publicity) have decided 
	 *   that cross site scripting is 'a bad thing' - so we don't expect a
	 *   response to this Remoting request. It will send the data to the server
	 *   and the iFrame will load as a child of the parent document, we just
	 *   won't be able to read it's contents. However, for saving data we will
	 *   assume that the server has managed to save the Label information.
	 *   Therefore this is suitable client->server XSS only.
	 */
	fnStateServerSave: function (  )
	{
		var iRequest = Remoting.fnNewRequest( 
			"http://www.sprymedia.co.uk/design/label/media/php/label_save.php", 
			function(){ ; } );
		
		Remoting.fnAddPost( iRequest, 'sBrowserCode', this.sBrowserStoreCode );
		Remoting.fnAddPost( iRequest, 'sUserShareCode', this.sUserShareCode );
		Remoting.fnAddPost( iRequest, 'sUrl', window.location.href );
		Remoting.fnAddPost( iRequest, 'oLabels', DesignCore.toJsonString(this.aoLabels) );
		Remoting.fnRequest( iRequest );
	},
	
	
	/*
	 * Function: fnStateLoad
	 * Purpose:  Load a saved save of Labels
	 * Returns:  int: -1 on no cookie, 0 on loaded
	 * Inputs:   -
	 * Notes:    Like the save state function, the load state also falls foul of
	 *   XSS limitations in browsers. However, for server -> client comms we
	 *   use a PHP script which will output Javascript with the required calls
	 *   and information needed as javascript variables. So here we simply add
	 *   a 'script' tag, with a few GET variables and that's that. Note that
	 *   the limit of GET is 2048 chars (IE) - however this is plenty good 
	 *   enough to send the browser code and the share code
	 */
	fnStateLoad: function ()
	{
		if ( this.sBrowserStoreCode != '' || this.sUserShareCode != '' )
		{
			/* Load the labels state from the server */
			var nScript = document.createElement( 'script' );
			nScript.type = 'text/javascript';
			nScript.language = 'JavaScript';
			nScript.src = "http://www.sprymedia.co.uk/design/label/media/php/label_load.php?"+
				'sBrowserCode='+this.sBrowserStoreCode+
				'&sUserShareCode='+this.sUserShareCode+
				'&sUrl='+window.location.href+
				'&fnCallback=Label.fnStateLoaded';
			document.body.appendChild( nScript );
		}
		else
		{
			/* Else reutrn an error so init can generate a code and get on with it */
			return -1;
		}
	},
	
	
	/*
	 * Function: fnStateLoaded
	 * Purpose:  Callback function for using javascript to load a Labels state
	 * Returns:  -
	 * Inputs:   array:aoLabels - labels object that was previously saved
	 *           string:sIndicator - information about the loading process - opt
	 *             'UNKNOWN' - unknown code
	 */
	fnStateLoaded: function ( aoLabels, sIndicator )
	{
		if ( typeof sIndicator != 'undefined' && sIndicator == 'UNKNOWN' )
		{
			if ( this.aoLabels.length == 0 )
			{
				this.fnCreateLabel();
			}
			var nDetails = DesignCore.fnGetElementsByClassName('label_info_details', 'div', document )[0];
			nDetails.innerHTML = 'The code you entered is unknown';
		}
		else if ( typeof aoLabels != 'undefined' && aoLabels != null && aoLabels.length != 0 )
		{
			/* Remove any old labels first */
			var iLength = this.aoLabels.length;
			for ( var i=iLength-1 ; i>=0 ; i-- )
			{
				nLabel = document.getElementById( this.aoLabels[i].sId );
				nLabel.parentNode.removeChild( nLabel );
				this.aoLabels.splice( i, 1 );
			}
			
			
			/* Load new labels */
			this.aoLabels = aoLabels;
			for ( var i=0 ; i<this.aoLabels.length ; i++ )
			{
				if ( this.aoLabels[ i ].iZIndex > this.iTopZIndex )
				{
					this.iTopZIndex = this.aoLabels[ i ].iZIndex;
				}
				this.fnCreateLabel( i )
			}
		}
		else
		{
			/* No saved state - so let's make a new one */
			this.fnCreateLabel();
		}
		
		if ( document.getElementById('LoadingLabel') )
		{
			document.body.removeChild( document.getElementById('LoadingLabel') );
		}
	},
	
	
	
	
	/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
	 * General control
	 */
	
	/*
	 * Function: fnDetails
	 * Purpose:  Deal with a click on the second (but primary) line of a label's
	 *   controls
	 * Returns:  -
	 * Inputs:   e:event
	 */
	fnDetails: function ( e )
	{
		var nTarget = this.fnGetElementFromEvent( e );
		var nContainer = this.fnFindParent( nTarget, "DIV", 'label_content_wrapper' );
		var nDetails = DesignCore.fnGetElementsByClassName('label_info_details', 'div', nContainer )[0];
		var sLabelId = nContainer.getAttribute('id');
		var iLabelIndex = this.fnGetLabelIndex( sLabelId );
		
		if ( e.stopPropagation ) // assume w3c
		{
			e.preventDefault();
			e.stopPropagation();
		}
		else // assume MS
		{
			e.returnValue = false;
			e.cancelBubble = true;
		}
		
		/* Remove listeners from a previous round - if there are any */
		jsCore.removeListener( { 'sLabel': 'Label_Options_'+sLabelId } );
		
		switch ( nTarget.className )
		{
			/*
			 * Create a new label
			 */
			case 'label_info_new':
				this.fnCreateLabel( null, iLabelIndex );
				this.fnStateSave( false );
				break;
			
			/*
			 * Display options for changing the display of a label
			 */
			case 'label_info_options':
				/* Remove the old content if it is the second click */
				if ( nDetails.innerHTML.match('Colour') )
				{
					nDetails.innerHTML = '&nbsp;';
					break;
				}
				
				nDetails.innerHTML = '\
					<div class="label_float_left">Colour:&nbsp;</div>\
					<div class="label_colour label_black"></div>\
					<div class="label_colour label_red"></div>\
					<div class="label_colour label_green"></div>\
					<div class="label_colour label_blue"></div>\
					<div class="label_colour label_yellow"></div>\
					<div class="label_colour label_white"></div>\
					<div class="label_float_left"> &nbsp; - Size:&nbsp;</div> \
					<div class="label_float_left label_size_small">A&nbsp;</div> \
					<div class="label_float_left label_size_med">A&nbsp;</div> \
					<div class="label_float_left label_size_large">A </div> \
					<div class="label_float_clear">&nbsp;</div>';
					
				/* Controls */
				jsCore.addListener( {
					'mElement': nDetails,
					'sType': 'click',
					'fnCallback': this.fnOptions,
					'oObj': this,
					'mScope': true,
					'sLabel': 'Label_Options_'+sLabelId } );
				break;
			
			/*
			 * Share code
			 */
			case 'label_info_share':
				/* Remove the old content if it is the second click */
				if ( nDetails.innerHTML.match('Share code') )
				{
					nDetails.innerHTML = '&nbsp;';
					break;
				}
				
				nDetails.innerHTML = '\
					<div class="label_float_left">\
						Share code: \
					</div>\
					<div class="label_float_left">\
						<input type="text" name="sShareCode" value="Create a code">\
					</div>\
					<div class="label_float_left">\
						<div class="label_info_submit"></div>\
					</div>\
					<div class="label_float_clear">&nbsp;</div>';
				
				/* Controls */
				jsCore.addListener( {
					'mElement': nDetails,
					'sType': 'click',
					'fnCallback': this.fnShareCode,
					'oObj': this,
					'mScope': true,
					'sLabel': 'Label_Options_'+sLabelId } );
				
				jsCore.addListener( {
					'mElement': nDetails.getElementsByTagName('input')[0],
					'sType': 'keypress',
					'fnCallback': this.fnShareCode,
					'oObj': this,
					'mScope': true,
					'sLabel': 'Label_Options_'+sLabelId } );
				break;
			
			/*
			 * Load code
			 */
			case 'label_info_load':
				/* Remove the old content if it is the second click */
				if ( nDetails.innerHTML.match('Load code') )
				{
					nDetails.innerHTML = '&nbsp;';
					break;
				}
				
				nDetails.innerHTML = '\
					<div class="label_float_left">\
						Load code:\
					</div>\
					<div class="label_float_left">\
					 	<input type="text" name="sShareCode" value="Enter saved code">\
					</div>\
					<div class="label_float_left">\
						<div class="label_info_submit"></div>\
					</div>\
					<div class="label_float_clear">&nbsp;</div>';
				
				/* Controls */
				jsCore.addListener( {
					'mElement': nDetails,
					'sType': 'click',
					'fnCallback': this.fnCodeLoad,
					'oObj': this,
					'mScope': true,
					'sLabel': 'Label_Options_'+sLabelId } );
				
				jsCore.addListener( {
					'mElement': nDetails.getElementsByTagName('input')[0],
					'sType': 'keypress',
					'fnCallback': this.fnCodeLoad,
					'oObj': this,
					'mScope': true,
					'sLabel': 'Label_Options_'+sLabelId } );
				break;
			
			/*
			 * Delete label
			 */
			case 'label_info_delete':
				nContainer.parentNode.removeChild( nContainer );
				this.aoLabels.splice( iLabelIndex, 1 );
				this.fnStateSave( false );
				break;
			
			default:
				break;
		} /* switch */
	},
	
	
	/*
	 * Function: fnCodeLoad
	 * Purpose:  Handle user input and submit from the 'load code' field
	 * Returns:  -
	 * Inputs:   event:e
	 */
	fnCodeLoad: function ( e )
	{
		clearTimeout( this.toSaveTimer ); // Don't want to save stuff here
		var nTarget = this.fnGetElementFromEvent( e );
		var nContainer = this.fnFindParent( nTarget, "DIV", 'label_content_wrapper' );
		
		/* Clear the input field */
		if ( nTarget.nodeName == "INPUT" && nTarget.value == "Enter saved code" )
		{
			nTarget.value = "";
		}
		
		/* User has submitted the code */
		cKey = e.keyCode ? e.keyCode : e.which;
		if ( nTarget.className == "label_info_submit" || cKey == 13 )
		{
			var nInput = nContainer.getElementsByTagName('input')[0];
			this.sUserShareCode = nInput.value;
			this.fnStateLoad();
		}
	},
	
	
	/*
	 * Function: fnShareCode
	 * Purpose:  Handle user input and submit from the 'share code' field
	 * Returns:  -
	 * Inputs:   event:e
	 */
	fnShareCode: function ( e )
	{
		var nTarget = this.fnGetElementFromEvent( e );
		var nContainer = this.fnFindParent( nTarget, "DIV", 'label_content_wrapper' );
		
		/* Clear the input field */
		if ( nTarget.nodeName == "INPUT" && nTarget.value == "Create a code" )
		{
			nTarget.value = "";
		}
		
		/* User has submitted the code */
		cKey = e.keyCode ? e.keyCode : e.which;
		if ( nTarget.className == "label_info_submit" || cKey == 13 )
		{
			var nInput = nContainer.getElementsByTagName('input')[0];
			this.sUserShareCode = nInput.value;
			this.fnStateSave( false );
			
			var nDetails = DesignCore.fnGetElementsByClassName('label_info_details', 'div', nContainer )[0];
			nDetails.innerHTML = "Now sharing all labels using '"+ this.sUserShareCode +"'";
		}
	},
	
	
	/*
	 * Function: fnOptions
	 * Purpose:  Deal with a click on the options row - colour + size
	 * Returns:  -
	 * Inputs:   e:event
	 */
	fnOptions: function ( e )
	{
		if ( e.stopPropagation ) // assume w3c
		{
			e.preventDefault();
			e.stopPropagation();
		}
		else // assume MS
		{
			e.returnValue = false;
			e.cancelBubble = true;
		}
		
		var nTarget = this.fnGetElementFromEvent( e );
		var nContainer = this.fnFindParent( nTarget, "DIV", 'label_content_wrapper' );
		var nTextarea = nContainer.getElementsByTagName('textarea')[0];
		var iLabelIndex = this.fnGetLabelIndex( nContainer.getAttribute('id') );
		
		switch ( nTarget.className )
		{
			case 'label_colour label_black':
				nContainer.className = 'label_content_wrapper label_black';
				this.aoLabels[iLabelIndex].sColour = 'black';
				break;
			
			case 'label_colour label_red':
				nContainer.className = 'label_content_wrapper label_red';
				this.aoLabels[iLabelIndex].sColour = 'red';
				break;
			
			case 'label_colour label_blue':
				nContainer.className = 'label_content_wrapper label_blue';
				this.aoLabels[iLabelIndex].sColour = 'blue';
				break;
			
			case 'label_colour label_green':
				nContainer.className = 'label_content_wrapper label_green';
				this.aoLabels[iLabelIndex].sColour = 'green';
				break;
			
			case 'label_colour label_yellow':
				nContainer.className = 'label_content_wrapper label_yellow';
				this.aoLabels[iLabelIndex].sColour = 'yellow';
				break;
			
			case 'label_colour label_white':
				nContainer.className = 'label_content_wrapper label_white';
				this.aoLabels[iLabelIndex].sColour = 'white';
				break;
			
			case 'label_float_left label_size_small':
				nTextarea.style.fontSize = "11px";
				this.aoLabels[iLabelIndex].iSize = 11;
				break;
			
			case 'label_float_left label_size_med':
				nTextarea.style.fontSize = "14px";
				this.aoLabels[iLabelIndex].iSize = 14;
				break;
			
			case 'label_float_left label_size_large':
				nTextarea.style.fontSize = "18px";
				this.aoLabels[iLabelIndex].iSize = 18;
				break;
			
			default:
				break;
		} /* switch */
		
		this.fnStateSave( false );
	},
	
	
	/*
	 * Function: fnContentKeyUp
	 * Purpose:  Save the content of a label on ever keyup
	 * Returns:  -
	 * Inputs:   event:e
	 */
	fnContentKeyUp: function ( e )
	{
		/* Get the body tag for the target as it has the class/id we want */
		var nTarget = this.fnGetElementFromEvent( e );
		var nContainer = this.fnFindParent( nTarget, "DIV", 'label_content_wrapper' );
		var nTextarea = nContainer.getElementsByTagName('textarea')[0];
		var iLabelIndex = this.fnGetLabelIndex( nContainer.getAttribute('id') );
		
		this.aoLabels[iLabelIndex].sContent = nTextarea.value;
		this.fnStateSave( false );
	},
	
	
	
	/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
	 * Mouse
	 */
	
	/*
	 * Function: fnMouseDown
	 * Purpose:  Generic mouse down handler - deals with resize and movement
	 * Returns:  -
	 * Inputs:   e:event
	 */
	fnMouseDown: function ( e )
	{
		var bAddListeners = false;
		var nTarget = this.fnGetElementFromEvent( e );
		var nContainer = this.fnFindParent( nTarget, "DIV", 'label_content_wrapper' );
		var iLabelIndex = this.fnGetLabelIndex( nContainer.getAttribute('id') );
		
		if ( e.stopPropagation )
			e.stopPropagation();
		else
			e.cancelBubble = true;
		
		if ( nContainer.style.zIndex < this.iTopZIndex )
		{
			this.iTopZIndex++;
			nContainer.style.zIndex = this.iTopZIndex;
			this.aoLabels[ iLabelIndex ].iZIndex = this.iTopZIndex;
		}
		
		/* Label resize */
		if ( nTarget.className == 'label_resize' )
		{
			/* Prevent text selection for resize */
			if ( e.preventDefault )
				e.preventDefault();
			else
				e.returnValue = false;
			
			this.oDrag.sType = 'resize';
			this.oDrag.nTarget = nTarget.parentNode;
			this.oDrag.iLabelIndex = iLabelIndex;
			
			this.oDrag.iNodeStartWidth = this.oDrag.nTarget.offsetWidth;
			this.oDrag.iNodeStartHeight = this.oDrag.nTarget.offsetHeight;
			this.oDrag.iMouseStartX = e.clientX;
			this.oDrag.iMouseStartY = e.clientY;
			
			bAddListeners = true;
		}
		/* Label movement */
		else if ( nContainer != -1 && 
			nTarget.className != 'label_info_save' && // Ignore click options
			nTarget.className != 'label_info_options' &&
			nTarget.className != 'label_info_share' &&
			nTarget.className != 'label_info_delete'  )
		{
			this.oDrag.sType = 'move';
			this.oDrag.nTarget = nContainer;
			this.oDrag.iLabelIndex = iLabelIndex;
			
			this.oDrag.iNodeStartX = this.oDrag.nTarget.offsetLeft;
			this.oDrag.iNodeStartY = this.oDrag.nTarget.offsetTop;
			this.oDrag.iMouseStartX = e.clientX;
			this.oDrag.iMouseStartY = e.clientY;
			
			bAddListeners = true;
		}
		
		/* Add the listeners required for dragging */
		if ( bAddListeners )
		{
			jsCore.addListener( {
				'mElement': document,
			  'sType': 'mouseup',
			  'fnCallback': this.fnMouseRelease,
				'oObj': this,
				'mScope': true,
				'sLabel': 'Label' } );
			
			jsCore.addListener( {
				'mElement': document,
			  'sType': 'mousemove',
			  'fnCallback': this.fnMouseMove,
				'oObj': this,
				'mScope': true,
				'sLabel': 'Label' } );
		}
	},
	
	
	/*
	 * Function: fnMouseMove
	 * Purpose:  document mouse movement event handler
	 * Returns:  -
	 * Inputs:   e:event
	 */
	fnMouseMove: function ( e )
	{
		if ( this.oDrag.sType == 'resize' )
		{
			var iNewWidth = this.oDrag.iNodeStartWidth + (e.clientX - this.oDrag.iMouseStartX);
			var iNewHeight = this.oDrag.iNodeStartHeight + (e.clientY - this.oDrag.iMouseStartY);
			
			/* Stop rounding errors in safari */
			if ( navigator.userAgent.indexOf("WebKit") != -1 && iNewWidth%2 != 0 )
			{
				iNewWidth++;
			}
			
			// Simulate min-width for IE6 (just do it for everyone)
			if ( iNewWidth >= 150 )
			{
				this.oDrag.nTarget.style.width = iNewWidth+'px';
				this.aoLabels[ this.oDrag.iLabelIndex ].iWidth = iNewWidth+'px';
			}
			
			// Also simulate min-height
			if ( iNewHeight >= 50 )
			{
				this.oDrag.nTarget.style.height = iNewHeight+'px';
				this.aoLabels[ this.oDrag.iLabelIndex ].iHeight = iNewHeight+'px';
			}
		}
		else if ( this.oDrag.sType == 'move' )
		{
			var iTop = this.oDrag.iNodeStartY + e.clientY - this.oDrag.iMouseStartY;
			var iLeft = this.oDrag.iNodeStartX + e.clientX - this.oDrag.iMouseStartX;
			
			this.oDrag.nTarget.style.top = iTop+'px';
			this.aoLabels[ this.oDrag.iLabelIndex ].iTop = iTop+'px';
			
			this.oDrag.nTarget.style.left = iLeft+'px';
			this.aoLabels[ this.oDrag.iLabelIndex ].iLeft = iLeft+'px';
		}
	},
	
	
	/*
	 * Function: fnMouseRelease
	 * Purpose:  document mouse up event handler
	 * Returns:  -
	 * Inputs:   e:event
	 */
	fnMouseRelease: function ( e )
	{
		jsCore.removeListener( {
			mElement: document,
		  sType: 'mousemove',
		  fnCallback: this.fnMouseMove } );
		
		jsCore.removeListener( {
			mElement: document,
		  sType: 'mouseup',
		  fnCallback: this.fnMouseRelease } );
		
		/* Reset drag cache*/
		oDrag = {
			sType: null,
			nTarget: null,
			iLabelIndex: null,
			iNodeStartX: null,
			iNodeStartY: null,
			iNodeStartHeight: null,
			iNodeStartWidth: null,
			iMouseStartX: null,
			iMouseStartY: null
		};
		
		/* Save it all for later */
		this.fnStateSave( false );
	},
	
	
	
	/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
	 * Support
	 */
	
	/*
	 * Function: fnGetLabelIndex
	 * Purpose:  Find the index of a label in the control array
	 * Returns:  int:i - index we are looking for
	 * Inputs:   string:sId - id of the label we want the index of
	 */
	fnGetLabelIndex: function ( sId )
	{
		for ( var i=0 ; i<this.aoLabels.length ; i++ )
		{
			if ( this.aoLabels[i].sId == sId )
			{
				return i;
			}
		}
		
		return -1;
	},
	
	
	/*
	 * Function: fnCreateLabelObject
	 * Purpose:  Defaults for a new label
	 * Returns:  object:oLabel - standard object for label
	 * Inputs:   int:iParentIndex - index of the parent to this label - optional
	 */
	fnCreateLabelObject: function ( iParentIndex )
	{
		var oLabel = {
				sId: this.fnGenerateLabelID(),
				sContent: '',
				sColour: 'yellow',
				iSize: 14,
				iTop: document.documentElement && document.documentElement.scrollTop ?
					(document.documentElement.scrollTop+30)+'px' :
					(document.body.scrollTop+30)+'px',
				iLeft: '30px',
				iWidth: '150px',
				iHeight: '110px',
				iZIndex: (++this.iTopZIndex)
			};
		
		if ( typeof iParentIndex != 'undefined' && iParentIndex != null )
		{
			var iTop = parseInt( this.aoLabels[ iParentIndex ].iTop.replace('px','') );
			var iLeft = parseInt( this.aoLabels[ iParentIndex ].iLeft.replace('px','') );
			
			oLabel.iTop = (iTop+15)+'px';
			oLabel.iLeft = (iLeft+15)+'px';
			oLabel.iWidth = this.aoLabels[ iParentIndex ].iWidth;
			oLabel.iHeight = this.aoLabels[ iParentIndex ].iHeight;
		}
		
		return oLabel;
	},
	
	
	/*
	 * Function: fnGenerateLabelID
	 * Purpose:  Generate a semi random ID for Labels
	 * Returns:  string:sId - the id to use
	 * Inputs:   -
	 */
	fnGenerateLabelID: function()
	{
		var sId = "Label_"+ new Date().getTime();
		
		while ( document.getElementById(sId) )
		{
			sId = "Label_"+ new Date().getTime();
		}
		
		return sId;
	},
	
	
	/*
	 * Function: fnGenerateBrowserCode
	 * Purpose:  Generate a party random ID for the browser to store in a cookie
	 * Returns:  string: - the code to use
	 * Inputs:   -
	 */
	fnGenerateBrowserCode: function()
	{
		return (new Date().getTime() + Math.floor(Math.random()*101));
	},
	
	
	/*
	 * Function: fnGetElementFromEvent
	 * Purpose:  Return the target click node in a cross platform manner
	 * Returns:  node:nTarget
	 * Inputs:   event:e
	 */
	fnGetElementFromEvent: function ( e )
	{
		var nTarget;
		
		if (!e) 
			var e = window.event;
		if ( e == null )
			return null;
			
		if (e.target)
			nTarget = e.target;
		else if (e.srcElement)
			nTarget = e.srcElement;
	
		if (nTarget.nodeType == 3) // defeat Safari bug
			nTarget = nTarget.parentNode;
		
		return ( nTarget );
	},
	
	
	/*
	 * Function: fnFindParent
	 * Purpose:  Search up the DOM tree for a particular node
	 * Returns:  node:
	 * Inputs:   
	 *   node:nNode - The base search node
	 *   string:sTarget - Node name to look for
	 *   string:sClass - Class name to look for - options
	 */
	fnFindParent: function ( nNode, sTarget, sClass )
	{
		if ( sClass != undefined && sClass != null )
		{
			var sRegExp = new RegExp('\\b' + sClass + '\\b');
			if ( nNode.nodeName == sTarget && sRegExp.test(nNode.className) )
				return nNode;
			else if ( nNode.nodeName == 'HTML' )
				return -1;
			else
				return ( this.fnFindParent( nNode.parentNode, sTarget, sClass ) );
		}
		else
		{
			if ( nNode.nodeName == sTarget )
				return nNode;
			else if ( nNode.nodeName == 'HTML' )
				return -1;
			else
				return ( this.fnFindParent( nNode.parentNode, sTarget, null ) );
		}
	}
};

Label.fnPrerequisites();

}// If Label
